@@ -54,12 +54,13 @@ static bool dr_ptrn_compare_modify_hdr(size_t cur_num_of_actions,
 }
 
 static bool dr_ptrn_compare_pattern(enum dr_ptrn_type type,
+				    enum dr_ptrn_type cur_type,
 				    size_t cur_num_of_actions,
 				    __be64 cur_hw_action[],
 				    size_t num_of_actions,
 				    __be64 hw_action[])
 {
-	if (cur_num_of_actions != num_of_actions)
+	if ((cur_num_of_actions != num_of_actions) || (cur_type != type))
 		return false;
 
 	switch (type) {
@@ -87,6 +88,7 @@ dr_ptrn_find_cached_pattern(struct dr_ptrn_mngr *mngr,
 
 	list_for_each_safe(&mngr->ptrn_list, cached_pattern, tmp, list) {
 		if (dr_ptrn_compare_pattern(type,
+					    cached_pattern->type,
 					    cached_pattern->rewrite_param.num_of_actions,
 					    (__be64 *)cached_pattern->rewrite_param.data,
 					    num_of_actions,
@@ -101,8 +103,8 @@ dr_ptrn_find_cached_pattern(struct dr_ptrn_mngr *mngr,
 }
 
 static struct dr_ptrn_obj *
-dr_ptrn_alloc_pattern(struct dr_ptrn_mngr *mngr,
-		      uint16_t num_of_actions, uint8_t *data)
+dr_ptrn_alloc_pattern(struct dr_ptrn_mngr *mngr, uint16_t num_of_actions,
+		      uint8_t *data, enum dr_ptrn_type type)
 {
 	struct dr_ptrn_obj *pattern;
 	struct dr_icm_chunk *chunk;
@@ -135,6 +137,8 @@ dr_ptrn_alloc_pattern(struct dr_ptrn_mngr *mngr,
 		goto free_pattern;
 	}
 
+	pattern->type = type;
+
 	memcpy(pattern->rewrite_param.data, data, num_of_actions * DR_MODIFY_ACTION_SIZE);
 	pattern->rewrite_param.chunk = chunk;
 	pattern->rewrite_param.index = index;
@@ -178,7 +182,7 @@ dr_ptrn_cache_get_pattern(struct dr_ptrn_mngr *mngr,
 					      (__be64 *)data);
 	if (!pattern) {
 		/* Alloc and add new pattern to cache */
-		pattern = dr_ptrn_alloc_pattern(mngr, num_of_actions, data);
+		pattern = dr_ptrn_alloc_pattern(mngr, num_of_actions, data, type);
 		if (!pattern)
 			goto out_unlock;
 
